@@ -74,7 +74,7 @@ class KeyManager(object):
         pass
 
     @abc.abstractmethod
-    def get(self, context, managed_object_id):
+    def get(self, context, managed_object_id, metadata_only=False):
         """Retrieves the specified managed object.
 
         Implementations should verify that the caller has permissions to
@@ -82,6 +82,9 @@ class KeyManager(object):
         as context. If the user lacks permission then a NotAuthorized
         exception is raised.
 
+        If the caller requests only metadata, then the object that is
+        returned will contain only the secret metadata and no secret bytes.
+
         If the specified object does not exist, then a KeyError should be
         raised. Implementations should preclude users from discerning the
         UUIDs of objects that belong to other users by repeatedly calling
@@ -106,3 +109,17 @@ class KeyManager(object):
         considered "non-existent" and completely invisible.
         """
         pass
+
+    def list(self, context, object_type=None, metadata_only=False):
+        """Lists the managed objects given the criteria.
+
+        Implementations should verify that the caller has permission to list
+        the managed objects and should only list the objects the caller has
+        access to by checking the context object (context). A NotAuthorized
+        exception should be raised if the caller lacks permission.
+
+        A list of managed objects or managed object metadata should be
+        returned, depending on the metadata_only flag. If no objects are
+        found, an empty list should be returned instead.
+        """
+        return []
