@@ -20,6 +20,12 @@ try:
     from castellan.key_manager import barbican_key_manager as bkm
 except ImportError:
     bkm = None
+
+try:
+    from castellan.key_manager import vault_key_manager as vkm
+except ImportError:
+    vkm = None
+
 from castellan.common import utils
 
 _DEFAULT_LOG_LEVELS = ['castellan=WARN']
@@ -30,9 +36,12 @@ _DEFAULT_LOGGING_CONTEXT_FORMAT = ('%(asctime)s.%(msecs)03d %(process)d '
                                    '%(message)s')
 
 
-def set_defaults(conf, api_class=None, barbican_endpoint=None,
+def set_defaults(conf, backend=None, barbican_endpoint=None,
                  barbican_api_version=None, auth_endpoint=None,
-                 retry_delay=None, number_of_retries=None):
+                 retry_delay=None, number_of_retries=None, verify_ssl=None,
+                 api_class=None, vault_root_token_id=None, vault_url=None,
+                 vault_ssl_ca_crt_file=None, vault_use_ssl=None,
+                 barbican_endpoint_type=None):
     """Set defaults for configuration values.
 
     Overrides the default options values.
@@ -43,30 +52,61 @@ def set_defaults(conf, api_class=None, barbican_endpoint=None,
     :param auth_endpoint: Use this endpoint to connect to Keystone.
     :param retry_delay: Use this attribute to set retry delay.
     :param number_of_retries: Use this attribute to set number of retries.
+    :param verify_ssl: Use this to specify if ssl should be verified.
+    :param vault_root_token_id: Use this for the root token id for vault.
+    :param vault_url: Use this for the url for vault.
+    :param vault_use_ssl: Use this to force vault driver to use ssl.
+    :param vault_ssl_ca_crt_file: Use this for the CA file for vault.
+    :param barbican_endpoint_type: Use this to specify the type of URL.
+    :                              Valid values are: public, internal or admin.
     """
     conf.register_opts(km.key_manager_opts, group='key_manager')
     if bkm:
         conf.register_opts(bkm.barbican_opts, group=bkm.BARBICAN_OPT_GROUP)
+    if vkm:
+        conf.register_opts(vkm.vault_opts, group=vkm.VAULT_OPT_GROUP)
 
-    if api_class is not None:
-        conf.set_default('api_class', api_class, group='key_manager')
-    if bkm is not None and barbican_endpoint is not None:
-        conf.set_default('barbican_endpoint', barbican_endpoint,
-                         group=bkm.BARBICAN_OPT_GROUP)
-    if bkm is not None and barbican_api_version is not None:
-        conf.set_default('barbican_api_version', barbican_api_version,
-                         group=bkm.BARBICAN_OPT_GROUP)
-    if bkm is not None and auth_endpoint is not None:
-        conf.set_default('auth_endpoint', auth_endpoint,
-                         group=bkm.BARBICAN_OPT_GROUP)
+    # Use the new backend option if set or fall back to the older api_class
+    default_backend = backend or api_class
+    if default_backend is not None:
+        conf.set_default('backend', default_backend, group='key_manager')
 
-    if bkm is not None and retry_delay is not None:
-        conf.set_default('retry_delay', retry_delay,
-                         group=bkm.BARBICAN_OPT_GROUP)
+    if bkm is not None:
+        if barbican_endpoint is not None:
+            conf.set_default('barbican_endpoint', barbican_endpoint,
+                             group=bkm.BARBICAN_OPT_GROUP)
+        if barbican_api_version is not None:
+            conf.set_default('barbican_api_version', barbican_api_version,
+                             group=bkm.BARBICAN_OPT_GROUP)
+        if auth_endpoint is not None:
+            conf.set_default('auth_endpoint', auth_endpoint,
+                             group=bkm.BARBICAN_OPT_GROUP)
+        if retry_delay is not None:
+            conf.set_default('retry_delay', retry_delay,
+                             group=bkm.BARBICAN_OPT_GROUP)
+        if number_of_retries is not None:
+            conf.set_default('number_of_retries', number_of_retries,
+                             group=bkm.BARBICAN_OPT_GROUP)
+        if verify_ssl is not None:
+            conf.set_default('verify_ssl', verify_ssl,
+                             group=bkm.BARBICAN_OPT_GROUP)
+        if barbican_endpoint_type is not None:
+            conf.set_default('barbican_endpoint_type', barbican_endpoint_type,
+                             group=bkm.BARBICAN_OPT_GROUP)
 
-    if bkm is not None and number_of_retries is not None:
-        conf.set_default('number_of_retries', number_of_retries,
-                         group=bkm.BARBICAN_OPT_GROUP)
+    if vkm is not None:
+        if vault_root_token_id is not None:
+            conf.set_default('root_token_id', vault_root_token_id,
+                             group=vkm.VAULT_OPT_GROUP)
+        if vault_url is not None:
+            conf.set_default('vault_url', vault_url,
+                             group=vkm.VAULT_OPT_GROUP)
+        if vault_ssl_ca_crt_file is not None:
+            conf.set_default('ssl_ca_crt_file', vault_ssl_ca_crt_file,
+                             group=vkm.VAULT_OPT_GROUP)
+        if vault_use_ssl is not None:
+            conf.set_default('use_ssl', vault_use_ssl,
+                             group=vkm.VAULT_OPT_GROUP)
 
 
 def enable_logging(conf=None, app_name='castellan'):
@@ -102,4 +142,6 @@ def list_opts():
 
     if bkm is not None:
         opts.append((bkm.BARBICAN_OPT_GROUP, bkm.barbican_opts))
+    if vkm is not None:
+        opts.append((vkm.VAULT_OPT_GROUP, vkm.vault_opts))
     return opts
