@@ -54,6 +54,11 @@ class MockKeyManagerTestCase(test_key_mgr.KeyManagerTestCase):
 
         self.context = context.RequestContext('fake', 'fake')
 
+    def cleanUp(self):
+        super(MockKeyManagerTestCase, self).cleanUp()
+
+        self.key_mgr.keys = {}
+
     def test_create_key(self):
         key_id_1 = self.key_mgr.create_key(self.context)
         key_id_2 = self.key_mgr.create_key(self.context)
@@ -62,21 +67,24 @@ class MockKeyManagerTestCase(test_key_mgr.KeyManagerTestCase):
 
     def test_create_key_with_length(self):
         for length in [64, 128, 256]:
-            key_id = self.key_mgr.create_key(self.context, key_length=length)
+            key_id = self.key_mgr.create_key(self.context, length=length)
             key = self.key_mgr.get(self.context, key_id)
             self.assertEqual(length / 8, len(key.get_encoded()))
+            self.assertIsNotNone(key.id)
 
     def test_create_key_with_name(self):
         name = 'my key'
         key_id = self.key_mgr.create_key(self.context, name=name)
         key = self.key_mgr.get(self.context, key_id)
         self.assertEqual(name, key.name)
+        self.assertIsNotNone(key.id)
 
     def test_create_key_with_algorithm(self):
         algorithm = 'DES'
         key_id = self.key_mgr.create_key(self.context, algorithm=algorithm)
         key = self.key_mgr.get(self.context, key_id)
         self.assertEqual(algorithm, key.algorithm)
+        self.assertIsNotNone(key.id)
 
     def test_create_key_null_context(self):
         self.assertRaises(exception.Forbidden,
@@ -89,7 +97,9 @@ class MockKeyManagerTestCase(test_key_mgr.KeyManagerTestCase):
                 self.context, 'RSA', length, name=name)
 
             private_key = self.key_mgr.get(self.context, private_key_uuid)
+            self.assertIsNotNone(private_key.id)
             public_key = self.key_mgr.get(self.context, public_key_uuid)
+            self.assertIsNotNone(public_key.id)
 
             crypto_private_key = get_cryptography_private_key(private_key)
             crypto_public_key = get_cryptography_public_key(public_key)
@@ -148,6 +158,40 @@ class MockKeyManagerTestCase(test_key_mgr.KeyManagerTestCase):
         actual_key = self.key_mgr.get(self.context, key_id)
         self.assertEqual(_key, actual_key)
 
+        self.assertIsNotNone(actual_key.id)
+
+    def test_store_key_and_get_metadata(self):
+        secret_key = bytes(b'0' * 64)
+        _key = sym_key.SymmetricKey('AES', 64 * 8, secret_key)
+        key_id = self.key_mgr.store(self.context, _key)
+
+        actual_key = self.key_mgr.get(self.context,
+                                      key_id,
+                                      metadata_only=True)
+        self.assertIsNone(actual_key.get_encoded())
+        self.assertTrue(actual_key.is_metadata_only())
+
+        self.assertIsNotNone(actual_key.id)
+
+    def test_store_key_and_get_metadata_and_get_key(self):
+        secret_key = bytes(b'0' * 64)
+        _key = sym_key.SymmetricKey('AES', 64 * 8, secret_key)
+        key_id = self.key_mgr.store(self.context, _key)
+
+        actual_key = self.key_mgr.get(self.context,
+                                      key_id,
+                                      metadata_only=True)
+        self.assertIsNone(actual_key.get_encoded())
+        self.assertTrue(actual_key.is_metadata_only())
+
+        actual_key = self.key_mgr.get(self.context,
+                                      key_id,
+                                      metadata_only=False)
+        self.assertIsNotNone(actual_key.get_encoded())
+        self.assertFalse(actual_key.is_metadata_only())
+
+        self.assertIsNotNone(actual_key.id)
+
     def test_store_null_context(self):
         self.assertRaises(exception.Forbidden,
                           self.key_mgr.store, None, None)
@@ -173,3 +217,36 @@ class MockKeyManagerTestCase(test_key_mgr.KeyManagerTestCase):
     def test_delete_unknown_key(self):
         self.assertRaises(KeyError, self.key_mgr.delete, self.context,
                           None)
+
+    def test_list_null_context(self):
+        self.assertRaises(exception.Forbidden, self.key_mgr.list, None)
+
+    def test_list_keys(self):
+        key1 = sym_key.SymmetricKey('AES', 64 * 8, bytes(b'0' * 64))
+        self.key_mgr.store(self.context, key1)
+        key2 = sym_key.SymmetricKey('AES', 32 * 8, bytes(b'0' * 32))
+        self.key_mgr.store(self.context, key2)
+
+        keys = self.key_mgr.list(self.context)
+        self.assertEqual(2, len(keys))
+        self.assertTrue(key1 in keys)
+        self.assertTrue(key2 in keys)
+
+        for key in keys:
+            self.assertIsNotNone(key.id)
+
+    def test_list_keys_metadata_only(self):
+        key1 = sym_key.SymmetricKey('AES', 64 * 8, bytes(b'0' * 64))
+        self.key_mgr.store(self.context, key1)
+        key2 = sym_key.SymmetricKey('AES', 32 * 8, bytes(b'0' * 32))
+        self.key_mgr.store(self.context, key2)
+
+        keys = self.key_mgr.list(self.context, metadata_only=True)
+        self.assertEqual(2, len(keys))
+        bit_length_list = [key1.bit_length, key2.bit_length]
+        for key in keys:
+            self.assertTrue(key.is_metadata_only())
+            self.assertTrue(key.bit_length in bit_length_list)
+
+        for key in keys:
+            self.assertIsNotNone(key.id)
