@@ -15,9 +15,11 @@
 
 from oslo_config import cfg
 
+from castellan import key_manager
 from castellan.key_manager import barbican_key_manager as bkm
 from castellan import options
 from castellan.tests import base
+from castellan.tests.unit.key_manager import mock_key_manager
 
 
 class TestOptions(base.TestCase):
@@ -25,9 +27,15 @@ class TestOptions(base.TestCase):
     def test_set_defaults(self):
         conf = cfg.ConfigOpts()
 
-        api_class = 'test.api.class'
-        options.set_defaults(conf, api_class=api_class)
-        self.assertEqual(api_class, conf.key_manager.api_class)
+        self.assertTrue(isinstance(key_manager.API(conf),
+                                   bkm.BarbicanKeyManager))
+
+        cls = mock_key_manager.MockKeyManager
+        backend = '%s.%s' % (cls.__module__, cls.__name__)
+        options.set_defaults(conf, backend=backend)
+        self.assertEqual(backend, conf.key_manager.backend)
+        self.assertIsInstance(key_manager.API(conf),
+                              mock_key_manager.MockKeyManager)
 
         barbican_endpoint = 'http://test-server.org:9311/'
         options.set_defaults(conf, barbican_endpoint=barbican_endpoint)
@@ -39,7 +47,7 @@ class TestOptions(base.TestCase):
         self.assertEqual(barbican_api_version,
                          conf.get(bkm.BARBICAN_OPT_GROUP).barbican_api_version)
 
-        auth_endpoint = 'http://test-server.org:5000/'
+        auth_endpoint = 'http://test-server.org/identity'
         options.set_defaults(conf, auth_endpoint=auth_endpoint)
         self.assertEqual(auth_endpoint,
                          conf.get(bkm.BARBICAN_OPT_GROUP).auth_endpoint)
@@ -53,3 +61,13 @@ class TestOptions(base.TestCase):
         options.set_defaults(conf, number_of_retries=number_of_retries)
         self.assertEqual(number_of_retries,
                          conf.get(bkm.BARBICAN_OPT_GROUP).number_of_retries)
+
+        verify_ssl = True
+        options.set_defaults(conf, verify_ssl=True)
+        self.assertEqual(verify_ssl,
+                         conf.get(bkm.BARBICAN_OPT_GROUP).verify_ssl)
+
+        barbican_endpoint_type = 'internal'
+        options.set_defaults(conf, barbican_endpoint_type='internal')
+        result_type = conf.get(bkm.BARBICAN_OPT_GROUP).barbican_endpoint_type
+        self.assertEqual(barbican_endpoint_type, result_type)
